"""
Simple Echo Server capable of handling only one client at a time.
"""
import socket
import sys
import logging
logger = logging.getLogger(__name__)

# For debugging --
# import logging as logger
# logger.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class EchoServer:
    def __init__(self):
        logger.info('Initializing Simple Echo Server!')
        self.listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    # our connection handler.
    def handler(self, client_sock, client_address):
        """Handles the incoming client connection"""
        logger.info('client connected: ', client_address)
        while True:
            data = client_sock.recv(1024)
            logger.info('Received data: ', data)
            if data:
                client_sock.sendall(data)
            else:
                break

    def start(self, host, port):
        """Starts the Echo server"""
        conn = (host, port)
        # TCP connection
        self.listener.bind(conn)
        # how many connections do we want to allow? This would go in sock.listen
        self.listener.listen(16)
        while True:
            logger.info('EchoServer waiting for a connection on {}'.format(conn))
            try:
                client_sock, client_address = self.listener.accept()
                self.handler(client_sock, client_address)
            finally:
                self.stop()

    def stop(self):
        """Stops the Echo Server"""
        # Maybe shutdown the socket?
        self.listener.close()

# For infoging
if __name__ == '__main__':
    test = EchoServer()
    try:
        logger.info('Starting Echo Server!')
        test.start('', 4443)
    except KeyboardInterrupt:
        logger.info('Stopping Echo Server!')



"""WSGI server example"""
from gevent.pywsgi import WSGIServer
from http.server import BaseHTTPRequestHandler
import cgi
import io
import logging
logger = logging.getLogger(__name__)
html = """
<html>
<body>
   <form method="post" action="">
        <p>
           Age: <input type="text" name="age" value="%(age)s">
        </p>
        <p>
            Hobbies:
            <input
                name="hobbies" type="checkbox" value="software"
                %(checked-software)s
            > Software
            <input
                name="hobbies" type="checkbox" value="tunning"
                %(checked-tunning)s
            > Auto Tunning
        </p>
        <p>
            <input type="submit" value="Submit">
        </p>
    </form>
    <p>
        Age: %(age)s<br>
        Hobbies: %(hobbies)s
    </p>
</body>
</html>
"""

class HTTPHandler(BaseHTTPRequestHandler):
    def __init__(self, env, start_response):
        # the environment variable CONTENT_LENGTH may be empty or missing
        try:
            request_body_size = int(environ.get('CONTENT_LENGTH', 0))
        except (ValueError):
            request_body_size = 0

        # When the method is POST the variable will be sent
        # in the HTTP request body which is passed by the WSGI server
        # in the file like wsgi.input environment variable.
        request_body = environ['wsgi.input'].read(request_body_size)
        d = parse_qs(request_body)

        age = d.get('age', [''])[0] # Returns the first age value.
        hobbies = d.get('hobbies', []) # Returns a list of hobbies.

        # Always escape user input to avoid script injection
        age = escape(age)
        hobbies = [escape(hobby) for hobby in hobbies]

        response_body = html % { # Fill the above html template in
            'checked-software': ('', 'checked')['software' in hobbies],
            'checked-tunning': ('', 'checked')['tunning' in hobbies],
            'age': age or 'Empty',
            'hobbies': ', '.join(hobbies or ['No Hobbies?'])
        }

        status = '200 OK'

        response_headers = [
            ('Content-Type', 'text/html'),
            ('Content-Length', str(len(response_body)))
        ]

        start_response(status, response_headers)
        return [response_body]

class HTTPServer:
    def __init__(self):
        self.Handler = HTTPHandler
        self.listener = None

    def start(self, host, port):
        conn = (host, port)
        self.listener = WSGIServer(conn, self.Handler)
        try:
            logger.info('Starting HTTPServer on {}'.format(conn))
            self.listener.serve_forever()
        except:
            logger.debug('Exception Occurred!')
        finally:
            self.stop()

    def stop(self):
        self.listener.close()

if __name__ == '__main__':
    print('Serving on 8088...')



import multiprocessing
from setuptools import setup, find_packages
import iic4sem

setup(
    name='iic4sem',
    version=iic4sem.__version__,
    packages=find_packages(exclude=['*.pyc']),
    scripts=['bin/iic4sem'],
    url="https://github.com/xandfury/iic4sem",
    license='GPL 3',
    author="Abhinav Saxena",
    author_email="abhinav.saxena@iic.ac.in",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    project_urls={
        'Funding': 'http://iic.du.ac.in',
    },  
    keywords="IIC UDSC",
    include_package_data=True,
    long_description=open('README.md').read(),
    description="Basic Network Programming Assignments For IIC, UDSC 4 Sem Final",
    test_suite='nose.collector',
    tests_require="nose",
    install_requires=open('requirements.txt').read().splitlines()
)
